# Generated by Django 4.0.10 on 2023-06-24 09:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('authstats', '0006_report_show_character_image'),
    ]

    operations = [
        migrations.AddField(
            model_name='reportdatathrough',
            name='allow_sort',
            field=models.BooleanField(
                default=True, help_text='Allow sorting of this column in the UI.'),
        ),
    ]



from esi.openapi_clients import ESIClientProvider as OpenAPIProvider

from . import __title__, __url__, __version__

esi = OpenAPIProvider(
    compatibility_date="2025-11-06",
    ua_appname=__title__,
    ua_url=__url__,
    ua_version=__version__,
    operations=[
        "PostUniverseNames",
        "GetCorporationsCorporationIdMembers"
    ]
)



from __future__ import annotations

import json
import logging
from datetime import timedelta
from typing import List

from ninja import Field, NinjaAPI, Schema
from ninja.pagination import LimitOffsetPagination
from ninja.security import django_auth
from ninja.types import DictStrAny

from django.db.models import QuerySet
from django.utils import timezone

from allianceauth.eveonline.models import EveCorporationInfo

from authstats import schema
from authstats.tasks import run_report_for_corp
from authstats.utils import (
    find_unknown_character_ids, get_main_queryset, get_orphan_queryset,
)

from . import providers
from .models import AuthReportsConfiguration, Report, ReportResults

logger = logging.getLogger(__name__)


class Paginator(LimitOffsetPagination):
    class Input(Schema):
        limit: int = Field(1000000, ge=1)
        offset: int = Field(0, ge=0)

    def paginate_queryset(
        self,
        queryset: QuerySet,
        pagination: Input,
        **params: DictStrAny,
    ) -> any:
        offset = pagination.offset
        limit: int = pagination.limit
        return {
            "items": queryset[offset: offset + limit],
            "count": self._items_count(queryset),
        }


api = NinjaAPI(
    title="Auth Stats API",
    version="0.0.3",
    urls_namespace='authstats:api',
    auth=django_auth
)
# openapi_url=settings.DEBUG and "/openapi.json" or "")


@api.get(
    "/get_report/{report_id}/{corp_id}",
    response={200: dict, 403: str},
    tags=["Report"]
)
def get_report_for_corp(request, report_id: int, corp_id: int):
    if not AuthReportsConfiguration.get_solo().visible_corps_for_user(request.user).filter(corporation_id=corp_id).exists():
        return 403, f"Access Denied to get_report_for_corp {report_id}/{corp_id} for {request.user}"
    try:
        report = ReportResults.objects.get(
            corporation__corporation_id=corp_id, report_id=report_id)
        if report.report.restricted and not request.user.has_perm("authstats.restricted_reports"):
            return 403, f"Access Denied to get_report_for_corp {report_id}/{corp_id} for {request.user}"
        if report.last_update < timezone.now() - timedelta(hours=1):
            run_report_for_corp.delay(corp_id, report_id)
        return json.loads(report.results)
    except ReportResults.DoesNotExist:
        corp = EveCorporationInfo.objects.get(corporation_id=corp_id)
        report = Report.objects.get(id=report_id)
        # send task!
        run_report_for_corp.delay(corp_id, report_id)
        return {"report": {"name": f"{report.name}",
                           "corporation": corp.corporation_name},
                "data": False,
                "members": 0,
                "unknowns": 0}


@api.get(
    "/get_corps",
    response={200: List[schema.Corporation]},
    tags=["Report"]
)
def get_corps(request):
    return AuthReportsConfiguration.get_solo().visible_corps_for_user(request.user)


@api.get(
    "/get_reports",
    response={200: List[schema.Report], 403: str},
    tags=["Report"]
)
def get_reports(request):
    if not request.user.has_perm("authstats.basic_access"):
        return 403, f"Access Denied to get_reports for {request.user}"

    _reports = Report.objects.all()
    if not request.user.has_perm("authstats.restricted_reports"):
        _reports = _reports.filter(restricted=False)
    return _reports


def chunk_ids(lo, n=500):
    for i in range(0, len(lo), n):
        yield lo[i:i + n]


@api.get(
    "/get_unknowns/{corp_id}",
    response={200: dict, 403: str},
    tags=["Report"]
)
def get_orphans_for_corp(request, corp_id: int):
    if not AuthReportsConfiguration.get_solo().visible_corps_for_user(request.user).filter(corporation_id=corp_id).exists():
        return 403, f"Access Denied to get_orphans_for_corp for {request.user}"

    corp = EveCorporationInfo.objects.get(corporation_id=corp_id)

    mains = get_main_queryset(corp_id)
    unknown_member_list = find_unknown_character_ids(corp_id)
    orphan_characters = get_orphan_queryset(corp_id)

    names = []
    for c in chunk_ids(list(unknown_member_list)):
        try:
            for _n in providers.esi.client.Universe.PostUniverseNames(
                body=c
            ).result(use_etag=False):
                names.append(
                    {
                        "character": {
                            "id": _n.id,
                            "name": _n.name
                        }
                    }
                )
        except Exception as e:
            logger.exception(e)

    orphans = []
    for c in orphan_characters:
        try:
            orphans.append(
                {
                    "character": {
                        "id": c.character_id,
                        "name": c.character_name
                    },
                    "main": {
                        "id": c.character_ownership.user.profile.main_character.character_id,
                        "name": c.character_ownership.user.profile.main_character.character_name,
                        "cid": c.character_ownership.user.profile.main_character.corporation_id,
                        "cname": c.character_ownership.user.profile.main_character.corporation_name
                    }
                }
            )
        except Exception as e:
            logger.exception(e)

    return {
        "missing": {
            "name": "Missing Characters",
            "corporation": corp.corporation_name,
            "members": mains.count(),
            "data": names
        },
        "orphans": {
            "name": "Orphaned Characters",
            "corporation": corp.corporation_name,
            "members": mains.count(),
            "data": orphans
        }
    }



import json
import logging

from celery import shared_task

from django.contrib.auth.models import User
from django.core.serializers.json import DjangoJSONEncoder
from django.utils import timezone

from allianceauth.eveonline.models import EveCorporationInfo
from allianceauth.services.tasks import QueueOnce

from authstats.utils import (
    find_unknown_character_ids, get_main_queryset, get_orphan_queryset,
)

from .models import Report, ReportDataThrough, ReportResults

TZ_STRING = "%Y-%m-%dT%H:%M:%SZ"

logger = logging.getLogger(__name__)


@shared_task(bind=True, base=QueueOnce, max_retries=None)
def run_report_for_corp(self, corp_id, report_id):
    report = Report.objects.get(id=report_id)
    corp = EveCorporationInfo.objects.get(corporation_id=corp_id)
    fields = ReportDataThrough.objects.filter(report=report).order_by("rank")

    mains = get_main_queryset(corp_id)
    unknown_character_count = len(find_unknown_character_ids(corp_id))
    orphan_character_count = get_orphan_queryset(corp_id)

    output = {"report": {"name": report.name,
                         "corporation": corp.corporation_name},
              "headers": {},
              "data": {},
              "members": mains.count(),
              "unknowns": unknown_character_count + orphan_character_count,
              "updated": timezone.now(),
              "show_avatar": report.show_character_image
              }

    for f in fields:
        output['headers'][f.id] = {"rank": f.rank, "header": f.header,
                                   "field": f"field-{f.id}",
                                   "checkbox": f.checkbox_only,
                                   "aggregate": f.pass_fail_aggregate,
                                   "pass": 0,
                                   "description": f.long_descriptions,
                                   "allow_sort": f.allow_sort}

        try:  # TODO create new method for clean table data
            _f = f.report_Data_source.filter_object
            f_data = {}
            if hasattr(_f, "process_field"):
                f_data = _f.process_field(mains)
            elif hasattr(_f, "audit_filter"):
                logger.debug(
                    f"{_f} has no process_field, falling back to audit_filter")
                f_data = _f.audit_filter(mains)
            for u in mains:
                if u.id not in output['data']:
                    output['data'][u.id] = {"character": {
                        "name": u.profile.main_character.character_name, "id": u.profile.main_character.character_id}}
                output['data'][u.id][f"field-{f.id}"] = {
                    "check": f_data[u.id]['check'],
                    "data": "" if f.checkbox_only else f_data[u.id]['message']
                }
                output['headers'][f.id]['pass'] += 1 if f_data[u.id]['check'] else 0

        except Exception as e:
            logger.error(f"Error in Filter {e}")

    # strip the uid's from the data its just annoying to deal with.
    output['data'] = list(output['data'].values())
    output['headers'] = list(output['headers'].values())
    ReportResults.objects.update_or_create(corporation=corp, report=report,
                                           defaults={"results": json.dumps(output, cls=DjangoJSONEncoder)})

    return output


# @shared_task(bind=True, base=QueueOnce, max_retries=None)
# def run_reports_for_exporting(self, report_id, webhook_url):
#     """
#         Export the Stats for this report for all corps to a discord webhook as json?
#     """
#     pass...



import logging

from django.contrib.auth.models import User

from allianceauth.eveonline.models import EveCharacter
from esi.models import Token

from authstats import providers

logger = logging.getLogger(__name__)


def get_main_queryset(corp_id):
    return User.objects.filter(
        profile__main_character__corporation_id=corp_id
    )


def get_orphan_queryset(corp_id):
    return EveCharacter.objects.filter(
        corporation_id=corp_id,
        character_ownership__isnull=False,
    ).exclude(
        character_ownership__user__profile__main_character__corporation_id=corp_id
    )


def get_knowns_queryset(corp_id):
    return EveCharacter.objects.filter(
        corporation_id=corp_id,
        character_ownership__isnull=False,
    ).exclude(
        character_ownership__user__profile__main_character__isnull=True
    )


def find_unknown_character_ids(corp_id):
    known_character_ids = get_knowns_queryset(corp_id).values_list(
        "character_id",
        flat=True
    )
    unknown_member_list = []
    try:
        scopes = ["esi-corporations.read_corporation_membership.v1"]

        token = Token.objects.filter(
            character_id__in=EveCharacter.objects.filter(
                corporation_id=corp_id
            ).values(
                'character_id'
            )
        ).require_scopes(scopes)

        if not token.exists():
            raise Exception("No Tokens")

        unknown_member_list = set(
            providers.esi.client.Corporation.GetCorporationsCorporationIdMembers(
                corporation_id=corp_id,
                token=token.first(),
            ).result(
                use_etag=False
            )
        )
        for m in known_character_ids:
            try:
                unknown_member_list.remove(m)
            except KeyError:
                pass
    except Exception as e:
        logger.exception(e)

    return unknown_member_list



from django.urls import include, path, re_path

from . import app_settings, views
from .api import api

app_name = 'authstats'

urlpatterns = [
    re_path(r'^api/', api.urls),
    re_path(r'^add_corp_token/', views.add_corp, name="add_corp"),
    path('show/', views.react_redirect, name='base'),
    path('show/<int:cid>/<int:rid>/', views.react_main, name='report'),
]



from django.contrib.contenttypes.models import ContentType
from django.core.management.base import BaseCommand

from ...models import ReportDataSource
from ...signals import filters


class Command(BaseCommand):
    help = 'Syncing all the Models from Secure Group Filters'

    def handle(self, *args, **options):
        self.stdout.write("Syncing all the Models from Secure Group Filters")
        for _filter in filters.get_hooks():
            for filter in _filter.objects.all():
                if not ReportDataSource.objects.filter(
                        content_type=ContentType.objects.get_for_model(
                            filter).id,
                        object_id=filter.id).exists():
                    ReportDataSource.objects.create(filter_object=filter)



from django.contrib.auth.decorators import login_required, permission_required
from django.shortcuts import redirect, render
from django.template import TemplateDoesNotExist
from django.utils.translation import gettext_lazy as _

from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo
from esi.decorators import token_required

from authstats.models import Report

from . import __version__

REQUIRED_SCOPES = [
    "esi-corporations.read_corporation_membership.v1"
]


@login_required
@token_required(scopes=REQUIRED_SCOPES)
def add_corp(request, token):
    char = EveCharacter.objects.get_character_by_id(token.character_id)
    EveCorporationInfo.objects.get_or_create(
        corporation_id=char.corporation_id,
        defaults={
            'member_count': 0,
            'corporation_ticker': char.corporation_ticker,
            'corporation_name': char.corporation_name
        }
    )
    return redirect('authstats:base')


@permission_required("authstats.basic_access")
def react_main(request, rid, cid):
    try:
        return render(
            request,
            'authstats/react_base_bs5.html',
            context={
                "version": __version__,
                "app_name": "authstats",
                "page_title": "Auth Reports"
            }
        )
    except TemplateDoesNotExist:
        return render(
            request,
            'authstats/react_base.html',
            context={
                "version": __version__,
                "app_name": "authstats",
                "page_title": "Auth Reports"
            }
        )


@permission_required("authstats.basic_access")
def react_redirect(request):
    # get available models
    rid = 0
    try:
        rid = Report.objects.all().first().id
    except Exception:
        pass

    cid = request.user.profile.main_character.corporation_id
    return redirect("authstats:report", rid=rid, cid=cid)



# Generated by Django 4.0.10 on 2023-06-12 12:29

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('eveonline', '0017_alliance_and_corp_names_are_not_unique'),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Report',
            fields=[
                ('id', models.AutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=250)),
            ],
        ),
        migrations.CreateModel(
            name='ReportDataSource',
            fields=[
                ('id', models.AutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField(editable=False)),
                ('content_type', models.ForeignKey(editable=False,
                 on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype')),
            ],
            options={
                'default_permissions': [],
            },
        ),
        migrations.CreateModel(
            name='ReportFieldAlts',
            fields=[
                ('id', models.AutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=500)),
                ('description', models.CharField(max_length=500)),
                ('show_out_of_corp_alts', models.BooleanField(default=True)),
            ],
            options={
                'verbose_name': 'Report Field: Character Alts',
                'verbose_name_plural': 'Report Field: Character Alts',
            },
        ),
        migrations.CreateModel(
            name='ReportResults',
            fields=[
                ('id', models.AutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('results', models.TextField()),
                ('last_update', models.DateTimeField(auto_now=True)),
                ('corporation', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='eveonline.evecorporationinfo')),
                ('report', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='authstats.report')),
            ],
        ),
        migrations.CreateModel(
            name='ReportDataThrough',
            fields=[
                ('id', models.AutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('rank', models.IntegerField(default=5)),
                ('header', models.CharField(max_length=250)),
                ('pass_fail_aggregate', models.BooleanField(default=False)),
                ('report', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='authstats.report')),
                ('report_Data_source', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='authstats.reportdatasource')),
            ],
            options={
                'verbose_name': 'Report Field',
                'verbose_name_plural': 'Report Fields',
            },
        ),
        migrations.AddField(
            model_name='report',
            name='report_fields',
            field=models.ManyToManyField(
                through='authstats.ReportDataThrough', to='authstats.reportdatasource'),
        ),
        migrations.CreateModel(
            name='AuthReportsConfiguration',
            fields=[
                ('id', models.AutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('holding_corps', models.ManyToManyField(
                    to='eveonline.evecorporationinfo')),
            ],
            options={
                'verbose_name': 'Site Configuration',
                'permissions': (('holding_corps', 'Can access configured holding corp reports.'), ('own_corp', 'Can access own corporations reports.'), ('own_alliance', 'Can access own alliances reports.'), ('own_state', 'Can access own states reports.')),
                'default_permissions': [],
            },
        ),
    ]



# Generated by Django 4.0.10 on 2023-06-24 09:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('authstats', '0004_alter_authreportsconfiguration_options'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='authreportsconfiguration',
            options={'default_permissions': [], 'permissions': (('basic_access', 'Can access reports module'), ('holding_corps', 'Can access configured holding corp reports.'), ('own_corp', 'Can access own corporations reports.'), (
                'own_alliance', 'Can access own alliances reports.'), ('own_state', 'Can access own states reports.'), ('restricted_reports', 'Can access restricted reports.')), 'verbose_name': 'Auth Reports Configuration'},
        ),
        migrations.AddField(
            model_name='report',
            name='restricted',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='reportdatathrough',
            name='long_descriptions',
            field=models.CharField(
                blank=True, default=None, help_text='Long description show to the user on hover of column name.', max_length=500, null=True),
        ),
        migrations.AlterField(
            model_name='reportdatathrough',
            name='checkbox_only',
            field=models.BooleanField(
                default=False, help_text='Show this columns data as pass/fail checkboxes only.'),
        ),
        migrations.AlterField(
            model_name='reportdatathrough',
            name='header',
            field=models.CharField(
                help_text='Column header show to the user.', max_length=250),
        ),
        migrations.AlterField(
            model_name='reportdatathrough',
            name='pass_fail_aggregate',
            field=models.BooleanField(
                default=False, help_text='Show an aggregate of how many members pass this field in the header.'),
        ),
        migrations.AlterField(
            model_name='reportdatathrough',
            name='rank',
            field=models.IntegerField(
                default=5, help_text='Order the field will be show in. Lowest First.'),
        ),
    ]



"""
    Configurable Report Framework for corporate level aggregation of any data
"""
__version__ = "0.1.0"
__title__ = "AuthStats"
__url__ = "https://github.com/Solar-Helix-Independent-Transport/allianceauth-auth-reports"



from ..models import AuthReportsConfiguration
from . import UserTestCase


class TestCorptoolsCorpAccessPerms(UserTestCase):
    def test_no_perms_get_self_u1(self):
        cs = AuthReportsConfiguration.get_solo().visible_corps_for_user(self.user1)
        self.assertNotIn(self.corp1, cs)
        self.assertNotIn(self.corp2, cs)
        self.assertNotIn(self.corp3, cs)
        self.assertNotIn(self.corp4, cs)

    def test_no_perms_get_self_u2(self):
        cs = AuthReportsConfiguration.get_solo().visible_corps_for_user(self.user2)
        self.assertNotIn(self.corp1, cs)
        self.assertNotIn(self.corp2, cs)
        self.assertNotIn(self.corp3, cs)
        self.assertNotIn(self.corp4, cs)

    def test_no_perms_get_self_u3(self):
        cs = AuthReportsConfiguration.get_solo().visible_corps_for_user(self.user2)
        self.assertNotIn(self.corp1, cs)
        self.assertNotIn(self.corp2, cs)
        self.assertNotIn(self.corp3, cs)
        self.assertNotIn(self.corp4, cs)

    def test_get_corp_u1(self):
        self.user1.user_permissions.add(self.own_corp)
        self.user1.refresh_from_db()
        cs = AuthReportsConfiguration.get_solo().visible_corps_for_user(self.user1)
        self.assertIn(self.corp1, cs)
        self.assertNotIn(self.corp2, cs)
        self.assertNotIn(self.corp3, cs)
        self.assertNotIn(self.corp4, cs)

    def test_get_corp_u2(self):
        self.user2.user_permissions.add(self.own_corp)
        self.user2.refresh_from_db()
        cs = AuthReportsConfiguration.get_solo().visible_corps_for_user(self.user2)
        self.assertNotIn(self.corp1, cs)
        self.assertIn(self.corp2, cs)
        self.assertNotIn(self.corp3, cs)
        self.assertNotIn(self.corp4, cs)

    def test_get_corp_u3(self):
        self.user3.user_permissions.add(self.own_corp)
        self.user3.refresh_from_db()
        cs = AuthReportsConfiguration.get_solo().visible_corps_for_user(self.user3)
        self.assertNotIn(self.corp1, cs)
        self.assertNotIn(self.corp2, cs)
        self.assertIn(self.corp3, cs)
        self.assertNotIn(self.corp4, cs)

    def test_get_alliance_u1(self):
        self.user1.user_permissions.add(self.own_alliance)
        self.user1.refresh_from_db()
        cs = AuthReportsConfiguration.get_solo().visible_corps_for_user(self.user1)
        self.assertIn(self.corp1, cs)
        self.assertNotIn(self.corp2, cs)
        self.assertNotIn(self.corp3, cs)
        self.assertNotIn(self.corp4, cs)

    def test_get_alliance_u2(self):
        self.user2.user_permissions.add(self.own_alliance)
        self.user2.refresh_from_db()
        cs = AuthReportsConfiguration.get_solo().visible_corps_for_user(self.user2)
        self.assertNotIn(self.corp1, cs)
        self.assertIn(self.corp2, cs)
        self.assertNotIn(self.corp3, cs)
        self.assertNotIn(self.corp4, cs)

    def test_get_alliance_u3(self):
        self.user3.user_permissions.add(self.own_alliance)
        self.user3.refresh_from_db()
        cs = AuthReportsConfiguration.get_solo().visible_corps_for_user(self.user3)
        self.assertNotIn(self.corp1, cs)
        self.assertNotIn(self.corp2, cs)
        self.assertIn(self.corp3, cs)
        self.assertNotIn(self.corp4, cs)  # alt corp no show
